/** a new cdc sample code

    based on the original CDC.C from WCA this version is intended
    for the CH552 device with several changes to the original code.
    The Original code compiles but has 7 linkerwarnings about unused
    code and uses the large memory model. Its size is about 4000 bytes.
    While the basic structure of the code is still the same there are 
    several notable differences:

    - all warnings removed, memory model changed to small
    - all used functions from debug.c are copied into this file
    - a new h file (cdc12.h) with the CDC defines
    - readable descriptors using predefined constants
    - new code for using the unique chip serial as usb serial string
    - some small changes regarding rx and tx buffer handling -> still incomplete
    - some minor changes in init functions and some vars renamed
    - some rearangement of the XDATA vars.
    - changed fsys to 24 MHz at least the CH552 cant do USB at 32MHz
    - changed Config_Uart1 function not to use a param
    - reformated source without tabs

    With these changes the code size is about 2500 bytes when using
    linker code packing even less

    Todo: The delay function needs to be rewriten there are far too much #ifdefs
*/

#include ".\..\inc\ch552_keil.h"
#include ".\..\inc\mytypes.h"
#include ".\..\inc\usb\usb11.h"
#include ".\..\inc\usb\cdc12.h"

#include <string.h>

#define DEFAULT_ENDP0_SIZE 64
#define MAX_PACKET_SIZE    64

#define UART0_BUAD         57600
#define UART1_BUAD         57600
#define FREQ_SYS           24000000L

typedef struct _USB_SETUP_REQ
{
    uint8_t bRequestType;
    uint8_t bRequest;
    uint8_t wValueL;
    uint8_t wValueH;
    uint8_t wIndexL;
    uint8_t wIndexH;
    uint8_t wLengthL;
    uint8_t wLengthH;
}USB_SETUP_REQ, *PUSB_SETUP_REQ;

// globals
uint8_t xdata EP0_Buffer[DEFAULT_ENDP0_SIZE] _at_ 0x0000;
uint8_t xdata EP1_InBuffer [8]               _at_ 0x0040;
uint8_t xdata EP2_OutBuffer[MAX_PACKET_SIZE] _at_ 0x0080;
uint8_t xdata EP2_InBuffer [MAX_PACKET_SIZE] _at_ 0x00C0;
uint8_t xdata SerDes[32]                     _at_ 0x0100;
uint8_t xdata LineCoding[7]                  _at_ 0x0120;

uint16_t  SetupLen;
uint8_t   SetupReq,Count,UsbConfig;
uint8_t  *pDescr;


#define RXSIZE  64
uint8_t rxBuffer[RXSIZE];
uint8_t rdPtr;            // Uart_Input_Point = 0;
uint8_t wrPtr;            // Uart_Output_Point = 0;
uint8_t RxBytes;          // = 0;

uint8_t TxBytes;
uint8_t USBBufOutPoint;   // = 0;
bit UpPoint2_Busy;        //  = 0;

USB_SETUP_REQ   SetupReqBuf;
#define UsbSetupBuf     ((PUSB_SETUP_REQ)EP0_Buffer)


const uint8_t code DevDesc[18] =
{
   sizeof(DeviceDescriptor), USB_DEVICE_DESCRIPTOR,
   BCD_USB & 0xFF, BCD_USB >> 8,
   USB_CLASS_CDC,
   USB_SUBCLASS_UNDEFINED,
   USB_PROTOCOL_UNDEFINED,
   DEFAULT_ENDP0_SIZE,
   0x1A86 & 0xFF,0x1A86 >> 8,  // 0x86,0x1a,
   0x5722 & 0xFF,0x5722 >> 8,  // 0x22,0x57,
   0x0100 & 0xFF,0x0100 >> 8,  // 0x00,0x01,
   0x01,                       // vendorstr
   0x02,                       // devicestr
   0x03,                       // serialstr
   1
};

const uint8_t code CfgDesc[0x0043] =
{
   sizeof(ConfigurationDescriptor), USB_CONFIGURATION_DESCRIPTOR,
   sizeof(CfgDesc) & 0xFF, sizeof(CfgDesc) >> 8,
   USB_CLASS_CDC,
   1,                          // cfgvalue
   USB_STRING_UNDEFINED,       // no str
   0xa0,                       // buspwerd | remotewakeup
   100/2,                      // 100mA

   //interface 0
   sizeof(InterfaceDescriptor), USB_INTERFACE_DESCRIPTOR,
   0,                          // interface No 0
   0,                          // alternate setting
   1,                          // 1 extra endpoint
   USB_CLASS_CDC,              // class
   CDC_SUBCLASS_ABSTRACT,      // subclass
   0x01,                       // protokol
   USB_STRING_UNDEFINED,

   5,CDC_CS_INTERFACE,
   CDC_SUB_HEADER,             // identifier header
   BCD_CDC & 0xFF,BCD_CDC >> 8,//

   5,CDC_CS_INTERFACE,
   CDC_SUB_CALL_FUNCTION,      // identifier Call Management
   0x00,0x00,

   4,CDC_CS_INTERFACE,
   CDC_SUB_ABSTRACT_CTRL,      // identifier Abstract
   0x02,                       // cababilities

   5,CDC_CS_INTERFACE,
   CDC_SUB_UNION_FUNCTIONAL,   // identifier Functional
   0,                          // interface 0 ctrl
   1,                          // interface 1 Data

   sizeof(EndpointDescriptor),USB_ENDPOINT_DESCRIPTOR,
   1 | _HOST,                  // ep1 in
   EP_INTERRUPT,               // interrupt
   sizeof(EP1_InBuffer) & 0xFF,sizeof(EP1_InBuffer) >> 8,
   0xFF,                       // interval

   //interface1 data interface
   sizeof(InterfaceDescriptor),USB_INTERFACE_DESCRIPTOR,
   1,                          // interface No 1
   0,                          // alternate setting
   2,                          // 2 extra endpoint
   CDC_CLASS_DATA,
   USB_SUBCLASS_UNDEFINED,
   USB_PROTOCOL_UNDEFINED,
   USB_STRING_UNDEFINED,

   sizeof(EndpointDescriptor),USB_ENDPOINT_DESCRIPTOR,
   2 | _DEVICE,                // ep2 out
   EP_BULK,                    // bulk
   sizeof(EP2_InBuffer) & 0xFF,sizeof(EP2_InBuffer) >> 8,
   0x00,                       // interval

   sizeof(EndpointDescriptor),USB_ENDPOINT_DESCRIPTOR,
   2 | _HOST,                  // ep2 In
   EP_BULK,                    // bulk
   sizeof(EP2_OutBuffer) & 0xFF,sizeof(EP2_OutBuffer) >> 8,
   0x00                        // interval
};

const uint8_t code LangDes[4]=
{
   4,USB_STRING_DESCRIPTOR,
   0x09,0x04
};
/*
const uint8_t code SerDes[20]=
{
   20,USB_STRING_DESCRIPTOR,
   '2',0x00,
   '0',0x00,
   '1',0x00,
   '7',0x00,
   '-',0x00,
   '2',0x00,
   '-',0x00,
   '2',0x00,
   '5',0x00
};
*/

const uint8_t code Prod_Des[20]=
{
   20,USB_STRING_DESCRIPTOR,
   'C',0x00,
   'H',0x00,
   '5',0x00,
   '5',0x00,
   '2',0x00,
   '_',0x00,
   'C',0x00,
   'D',0x00,
   'C',0x00
};

const uint8_t code Manuf_Des[10]=
{
   10,USB_STRING_DESCRIPTOR,
   0x5F,0x6c,
   0xCF,0x82,
   0x81,0x6c,
   0x52,0x60
};


// copied from debug.c
void  CfgFsys(void)
{
   SAFE_MOD  = 0x55;
   SAFE_MOD  = 0xAA;
   CLOCK_CFG = CLOCK_CFG & ~ MASK_SYS_CK_SEL | 0x06;  // 24MHz
   SAFE_MOD  = 0x00;
}

void  mDelayuS(uint16_t n)
{
#ifdef  FREQ_SYS
#if   FREQ_SYS <= 6000000
    n >>= 2;
#endif
#if   FREQ_SYS <= 3000000
    n >>= 2;
#endif
#if   FREQ_SYS <= 750000
    n >>= 4;
#endif
#endif
  while (n)       // total = 12~13 Fsys cycles, 1uS @Fsys=12MHz
  {
    ++ SAFE_MOD;  // 2 Fsys cycles, for higher Fsys, add operation here
#ifdef  FREQ_SYS
#if   FREQ_SYS >= 14000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 16000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 18000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 20000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 22000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 24000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 26000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 28000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 30000000
    ++ SAFE_MOD;
#endif
#if   FREQ_SYS >= 32000000
    ++ SAFE_MOD;
#endif
#endif
    -- n;
  }
}

void  mDelaymS( uint16_t n )
{
  while (n--)
  {
     mDelayuS( 1000 );
  }
}

void  UART1Setup( )
{
   U1SM0    = 0;
   U1SMOD   = 1;
   U1REN    = 1;
   SBAUD1   = 0 - FREQ_SYS/16/UART1_BUAD;
   IE_UART1 = 1;
   EA = 1;
}

void UART1SendByte(uint8_t tx)
{
   SBUF1 = tx;
   while(U1TI ==0);
   U1TI = 0;
}

void InitLineCoding(void)
{
   LineCoding[0] = 0x00;
   LineCoding[1] = 0xe1;
   LineCoding[2] = 0x00;
   LineCoding[3] = 0x00;
   LineCoding[4] = 0x00;
   LineCoding[5] = 0x00;
   LineCoding[6] = 0x08;
}

/** convert a byte into 2 hex chars
    \param byte input
    \result converted Hex chars */
uint16_t ToHex(uint8_t byte)
{
   uint8_t  tmp;
   uint16_t result;

   tmp = byte >> 4;
   result  = ((tmp< 10) ? tmp+'0' : tmp+'A'-10) << 8;
   tmp = byte & 0x0F;
   result |= ((tmp< 10) ? tmp+'0' : tmp+'A'-10) & 0xFF;
   return result;
}

/** build a USB serial number string

    read the unique chip serial number and build a USB String from it */
void CreateSerialNoString(void)
{
   uint16_t hex;
   uint8_t  i = 0;
   uint16_t address = ROM_CHIP_ID_HX +1;
   SerDes[i++] = 22;   // strlen == 2 + 10 digits * sizeof(uint16_t)
   SerDes[i++] = USB_STRING_DESCRIPTOR;
   while  (i < 22)
   {
      hex= ToHex(CBYTE[address++]);
      SerDes[i++]  = HIGH (hex);
      SerDes[i++]  = 0;
      SerDes[i++]  = LOW  (hex);
      SerDes[i++]  = 0;
   }
}

void USBDeviceCfg()
{
   USB_CTRL    =  bUC_DEV_PU_EN | bUC_INT_BUSY | bUC_DMA_EN;
   USB_DEV_AD  = 0x00;
   UDEV_CTRL   = bUD_PD_DIS | bUD_PORT_EN;
}

void USBDeviceIntCfg()
{
   USB_INT_EN |= bUIE_SUSPEND | bUIE_TRANSFER | bUIE_BUS_RST;                                               //ʹ豸ж
   USB_INT_FG |= 0x1F; //????
   IE_USB = 1;
   EA = 1;
}

void USBDeviceEndPointCfg()
{
   UEP1_DMA   = EP1_InBuffer;
   UEP2_DMA   = EP2_OutBuffer;
   UEP2_3_MOD = 0xCC;
   UEP2_CTRL  = bUEP_AUTO_TOG | UEP_T_RES_NAK | UEP_R_RES_ACK;
   UEP1_CTRL  = bUEP_AUTO_TOG | UEP_T_RES_NAK;
   UEP0_DMA   = EP0_Buffer;
   UEP4_1_MOD = 0X40;
   UEP0_CTRL  = UEP_R_RES_ACK | UEP_T_RES_NAK;
}

void DeviceInit(void)
{
   if(SW1==0) CALL(0x3800);
   LED1=0;LED3=0;

   CreateSerialNoString();
   InitLineCoding();

   CfgFsys();
   mDelaymS(5);

   UART1Setup();

   USBDeviceCfg();
   USBDeviceEndPointCfg();
   USBDeviceIntCfg();

}

/*******************************************************************************
* Function Name  : Config_Uart1(UINT8 *cfg_uart)
* Description    : ô1
* Input          : ò λʡֹͣλУ顢λ
* Output         : None
* Return         : None
*******************************************************************************/
/*
void Config_Uart1(uint8_t *cfg_uart)
{
   uint32_t uart1_buad = 0;
   *((uint8_t *)&uart1_buad+0) = cfg_uart[3];
   *((uint8_t *)&uart1_buad+1) = cfg_uart[2];
   *((uint8_t *)&uart1_buad+2) = cfg_uart[1];
   *((uint8_t *)&uart1_buad+3) = cfg_uart[0];
   IE_UART1 = 0;
   SBAUD1   = 0 - FREQ_SYS/16/uart1_buad;
   IE_UART1 = 1;
}
*/

void Config_Uart1(void)
{
   uint32_t uart1_buad = 0;
   // swap the bytes   
   *((uint8_t *)&uart1_buad+0) = LineCoding[3];
   *((uint8_t *)&uart1_buad+1) = LineCoding[2];
   *((uint8_t *)&uart1_buad+2) = LineCoding[1];
   *((uint8_t *)&uart1_buad+3) = LineCoding[0];
   IE_UART1 = 0;
   SBAUD1   = 0 - FREQ_SYS/16/uart1_buad;
   IE_UART1 = 1;
}

void DeviceInterrupt (void) interrupt INT_NO_USB
{
   uint16_t len;

   if(UIF_TRANSFER)
   {
      switch (USB_INT_ST & (MASK_UIS_TOKEN | MASK_UIS_ENDP))
      {
         case UIS_TOKEN_IN | 1:
              {  // ep 1 linecoding
                 UEP1_T_LEN = 0;
                 UEP1_CTRL = UEP1_CTRL & ~ MASK_UEP_T_RES | UEP_T_RES_NAK;
              }
              break;
         case UIS_TOKEN_IN | 2:
              {  //ep 2 Data in
                 UEP2_T_LEN = 0;
                 UEP2_CTRL = UEP2_CTRL & ~ MASK_UEP_T_RES | UEP_T_RES_NAK;
                 UpPoint2_Busy = 0;
              }
              break;
         case UIS_TOKEN_OUT | 2:
              {  //ep2 Data out
                 if ( U_TOG_OK )
                 {
                    TxBytes = USB_RX_LEN;
                    USBBufOutPoint = 0;
                    UEP2_CTRL = UEP2_CTRL & ~ MASK_UEP_R_RES | UEP_R_RES_NAK;
                 }
              }
              break;
         case UIS_TOKEN_SETUP | 0:
              {  // control transfers
                 len = USB_RX_LEN;
                 if(len == (sizeof(USB_SETUP_REQ)))
                 {
                    SetupLen = ((uint16_t)UsbSetupBuf->wLengthH<<8) | (UsbSetupBuf->wLengthL);
                    len = 0;
                    SetupReq = UsbSetupBuf->bRequest;
                    if ((UsbSetupBuf->bRequestType & _REQUESTMASK) == _CLASS)
                    {  // class requ
                       switch( SetupReq )
                       {
                          case GET_LINE_CODING:   //0x21  currently configured
                               pDescr = LineCoding;
                               len = sizeof(LineCoding);
                               len = SetupLen >= DEFAULT_ENDP0_SIZE ? DEFAULT_ENDP0_SIZE : SetupLen;  // δ䳤
                               memcpy(EP0_Buffer,pDescr,len);
                               SetupLen -= len;
                               pDescr += len;
                               break;
                          case SET_CONTROL_LINE_STATE:  //0x22  generates RS-232/V.24 style control signals
                               break;
                          case SET_LINE_CODING:      //0x20  Configure
                               break;
                          default:
                               len = 0xFF;
                               break;
                       }
                    }
                    else if ((UsbSetupBuf->bRequestType & _REQUESTMASK) == _STANDART)
                    {  // std req
                       switch(SetupReq)
                       {
                          case USB_GET_DESCRIPTOR:
                               switch(UsbSetupBuf->wValueH)
                               {
                                  case USB_DEVICE_DESCRIPTOR:
                                       pDescr = DevDesc;
                                       len = sizeof(DevDesc);
                                       break;
                                  case USB_CONFIGURATION_DESCRIPTOR:
                                       pDescr = CfgDesc;
                                       len = sizeof(CfgDesc);
                                       break;
                                  case USB_STRING_DESCRIPTOR:
                                       if(UsbSetupBuf->wValueL == 0)
                                       {
                                          pDescr = LangDes;
                                          len = sizeof(LangDes);
                                       }
                                       else if(UsbSetupBuf->wValueL == 1)
                                       {
                                          pDescr = Manuf_Des;
                                          len = sizeof(Manuf_Des);
                                       }
                                       else if(UsbSetupBuf->wValueL == 2)
                                       {
                                          pDescr = Prod_Des;
                                          len = sizeof(Prod_Des);
                                       }
                                       else
                                       {
                                          pDescr = SerDes;
                                          len = 22;   //sizeof(SerDes);
                                       }
                                       break;
                                  default:
                                       len = 0xFF;
                                       break;
                               }
                               if (SetupLen > len)
                               {
                                  SetupLen = len;
                               }
                               len = SetupLen >= DEFAULT_ENDP0_SIZE ? DEFAULT_ENDP0_SIZE : SetupLen;
                               memcpy(EP0_Buffer,pDescr,len);
                               SetupLen -= len;
                               pDescr += len;
                               break;
                          case USB_SET_ADDRESS:
                               SetupLen = UsbSetupBuf->wValueL;
                               break;
                          case USB_GET_CONFIGURATION:
                               EP0_Buffer[0] = UsbConfig;
                               if ( SetupLen >= 1 )
                               {
                                   len = 1;
                               }
                               break;
                          case USB_SET_CONFIGURATION:
                               UsbConfig = UsbSetupBuf->wValueL;
                               break;
                          case USB_GET_INTERFACE:
                               break;
                          case USB_CLEAR_FEATURE:                                            //Clear Feature
                               if(( UsbSetupBuf->bRequestType & _RECIPIENTMASK) == _DEVICE )                  /* 豸 */
                               {
                                   if( ((( uint16_t)UsbSetupBuf->wValueH << 8 ) | UsbSetupBuf->wValueL ) == 0x01 )
                                   {
                                       if( CfgDesc[7] & 0x20 )
                                       {
                                           /*  */
                                       }
                                       else
                                       {
                                           len = 0xFF;                                        /* ʧ */
                                       }
                                   }
                                   else
                                   {
                                       len = 0xFF;                                            /* ʧ */
                                   }
                               }
                               else if ((UsbSetupBuf->bRequestType & _RECIPIENTMASK) == _ENDPOINT )// ˵
                               {
                                  switch( UsbSetupBuf->wIndexL & 0x0F)
                                  {
                                     case 0x03:
                                          //if (UsbSetupBuf->wIndexL & _HOST)  NAK(3,T);
                                          //else                               ACK(3,R);
                                          if (UsbSetupBuf->wIndexL & _HOST)
                                            UEP3_CTRL = UEP3_CTRL & ~ ( bUEP_T_TOG | MASK_UEP_T_RES ) | UEP_T_RES_NAK;
                                          else
                                            UEP3_CTRL = UEP3_CTRL & ~ ( bUEP_R_TOG | MASK_UEP_R_RES ) | UEP_R_RES_ACK;
                                         break;
                                     case 0x02:
                                          //if (UsbSetupBuf->wIndexL & _HOST)  NAK(2,T);
                                          //else                               ACK(2,R);
                                          if (UsbSetupBuf->wIndexL & _HOST)
                                             UEP2_CTRL = UEP2_CTRL & ~ ( bUEP_T_TOG | MASK_UEP_T_RES ) | UEP_T_RES_NAK;
                                          else
                                             UEP2_CTRL = UEP2_CTRL & ~ ( bUEP_R_TOG | MASK_UEP_R_RES ) | UEP_R_RES_ACK;
                                          break;
                                     case 0x01:
                                          //if (UsbSetupBuf->wIndexL & _HOST)  NAK(1,T);
                                          //else                               ACK(1,R);
                                          if (UsbSetupBuf->wIndexL & _HOST)
                                             UEP1_CTRL = UEP1_CTRL & ~ ( bUEP_T_TOG | MASK_UEP_T_RES ) | UEP_T_RES_NAK;
                                          else
                                             UEP1_CTRL = UEP1_CTRL & ~ ( bUEP_R_TOG | MASK_UEP_R_RES ) | UEP_R_RES_ACK;
                                          break;
                                     default:
                                         len = 0xFF;                                         // ֵ֧Ķ˵
                                         break;
                                  }
                               }
                               else
                               {
                                  len = 0xFF;                                                // Ƕ˵㲻֧
                               }
                               break;
                          case USB_SET_FEATURE:                                          /* Set Feature */
                               if((UsbSetupBuf->bRequestType & 0x1F) == _DEVICE)                  /* 豸 */
                               {
                                   if((((uint16_t)UsbSetupBuf->wValueH << 8 ) | UsbSetupBuf->wValueL ) == 0x01 )
                                   {
                                       if( CfgDesc[7] & 0x20 )
                                       {
                                           while ( XBUS_AUX & bUART0_TX )
                                           {
                                              ;
                                           }
                                           SAFE_MOD = 0x55;
                                           SAFE_MOD = 0xAA;
                                           WAKE_CTRL = bWAK_BY_USB | bWAK_RXD0_LO | bWAK_RXD1_LO;                      //USBRXD0/1źʱɱ
                                           PCON |= PD;                                                                 //˯
                                           SAFE_MOD = 0x55;
                                           SAFE_MOD = 0xAA;
                                           WAKE_CTRL = 0x00;
                                       }
                                       else
                                       {
                                           len = 0xFF;                                        /* ʧ */
                                       }
                                   }
                                   else
                                   {
                                       len = 0xFF;                                            /* ʧ */
                                   }
                               }
                               else if((UsbSetupBuf->bRequestType & 0x1F) == _ENDPOINT )             /* ö˵ */
                               {
                                   if((((uint16_t)UsbSetupBuf->wValueH << 8 ) | UsbSetupBuf->wValueL) == 0x00)
                                   {
                                      switch(((uint16_t)UsbSetupBuf->wIndexH << 8 ) | UsbSetupBuf->wIndexL )
                                      {
                                         case 0x83:
                                              UEP3_CTRL = UEP3_CTRL & (~bUEP_T_TOG) | UEP_T_RES_STALL;/* ö˵3 IN STALL */
                                              break;
                                         case 0x03:
                                              UEP3_CTRL = UEP3_CTRL & (~bUEP_R_TOG) | UEP_R_RES_STALL;/* ö˵3 OUT Stall */
                                              break;
                                         case 0x82:
                                              UEP2_CTRL = UEP2_CTRL & (~bUEP_T_TOG) | UEP_T_RES_STALL;/* ö˵2 IN STALL */
                                              break;
                                         case 0x02:
                                              UEP2_CTRL = UEP2_CTRL & (~bUEP_R_TOG) | UEP_R_RES_STALL;/* ö˵2 OUT Stall */
                                              break;
                                         case 0x81:
                                              UEP1_CTRL = UEP1_CTRL & (~bUEP_T_TOG) | UEP_T_RES_STALL;/* ö˵1 IN STALL */
                                              break;
                                         case 0x01:
                                              UEP1_CTRL = UEP1_CTRL & (~bUEP_R_TOG) | UEP_R_RES_STALL;/* ö˵1 OUT Stall */
                                         default:
                                              len = 0xFF;                                    /* ʧ */
                                              break;
                                      }
                                   }
                                   else
                                   {
                                      len = 0xFF;                                      /* ʧ */
                                   }
                               }
                               else
                               {
                                  len = 0xFF;                                          /* ʧ */
                               }
                               break;
                          case USB_GET_STATUS:
                               EP0_Buffer[0] = 0x00;
                               EP0_Buffer[1] = 0x00;
                               if ( SetupLen >= 2 )
                               {
                                   len = 2;
                               }
                               else
                               {
                                  len = SetupLen;
                               }
                               break;
                          default:
                               len = 0xff;                                                    //ʧ
                               break;
                       }
                    }
                    else 
                    {
                       len = 0xff;
                    } 
                 }
                 else
                 {
                    len = 0xff;
                 }
                 if(len == 0xff)
                 {
                    SetupReq = 0xFF;
                    UEP0_CTRL = bUEP_R_TOG | bUEP_T_TOG | UEP_R_RES_STALL | UEP_T_RES_STALL;//STALL
                 }
                 else if(len <= DEFAULT_ENDP0_SIZE)
                 {
                    UEP0_T_LEN = len;
                    UEP0_CTRL = bUEP_R_TOG | bUEP_T_TOG | UEP_R_RES_ACK | UEP_T_RES_ACK;
                 }
                 else
                 {
                    UEP0_T_LEN = 0;
                    UEP0_CTRL = bUEP_R_TOG | bUEP_T_TOG | UEP_R_RES_ACK | UEP_T_RES_ACK;
                 }
              }
              break;
         case UIS_TOKEN_IN | 0:
              {  //status out or datastage in
                 switch(SetupReq)
                 {
                    case USB_GET_DESCRIPTOR:
                         len = SetupLen >= DEFAULT_ENDP0_SIZE ? DEFAULT_ENDP0_SIZE : SetupLen;                                 //δ䳤
                         memcpy( EP0_Buffer, pDescr, len );                                   //ϴ
                         SetupLen -= len;
                         pDescr += len;
                         UEP0_T_LEN = len;
                         UEP0_CTRL ^= bUEP_T_TOG;                                             //ͬ־λת
                         break;
                    case USB_SET_ADDRESS:
                         USB_DEV_AD = USB_DEV_AD & bUDA_GP_BIT | SetupLen;
                         UEP0_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK;
                         break;
                    default:
                         UEP0_T_LEN = 0;                                                      //״̬׶жϻǿϴ0ݰƴ
                         UEP0_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK;
                         //break;
                 }
              }
              break;
         case UIS_TOKEN_OUT | 0:
              {  // status in or datastage out
                 if(SetupReq ==SET_LINE_CODING)
                 {
                    if( U_TOG_OK ) 
                    {
                       memcpy(LineCoding,UsbSetupBuf,USB_RX_LEN);
                       Config_Uart1();
                       UEP0_T_LEN = 0;
                       UEP0_CTRL |= UEP_R_RES_ACK | UEP_T_RES_ACK;        
                    }
                 }
                 else
                 {
                    UEP0_T_LEN = 0;  
                    UEP0_CTRL |= UEP_R_RES_ACK | UEP_T_RES_NAK; 
                 }
              }
//            break;
         default:
              break;
      }
      UIF_TRANSFER = 0;
   }

   if(UIF_BUS_RST)
   {
      UEP0_CTRL = UEP_R_RES_ACK | UEP_T_RES_NAK;
      UEP1_CTRL = bUEP_AUTO_TOG | UEP_T_RES_NAK;
      UEP2_CTRL = bUEP_AUTO_TOG | UEP_T_RES_NAK | UEP_R_RES_ACK;
      USB_DEV_AD = 0x00;
      UIF_SUSPEND = 0;
      UIF_TRANSFER = 0;
      UIF_BUS_RST = 0;
      rdPtr = 0;
      wrPtr = 0;
      RxBytes = 0;
      TxBytes = 0;
      UsbConfig = 0;
      UpPoint2_Busy = 0;
   }

   if (UIF_SUSPEND)
   {
       UIF_SUSPEND = 0;
       if ( USB_MIS_ST & bUMS_SUSPEND )
       {
           while ( XBUS_AUX & bUART0_TX )
           {
               ;
           }
           SAFE_MOD = 0x55;
           SAFE_MOD = 0xAA;
           WAKE_CTRL = bWAK_BY_USB | bWAK_RXD0_LO | bWAK_RXD1_LO;
           PCON |= PD;
           SAFE_MOD = 0x55;
           SAFE_MOD = 0xAA;
           WAKE_CTRL = 0x00;
       }
   }
   else
   {
      USB_INT_FG = 0xFF;
   }
}

void Uart1_ISR(void) interrupt INT_NO_UART1
{
   if(U1RI)
   {
      rxBuffer[rdPtr++] = SBUF1;
      RxBytes++;
      rdPtr &= 0x3F; // limit the length
      U1RI =0;
   }
}


main()
{
   uint8_t len;
   uint8_t timeout = 0;

   DeviceInit();

   UEP0_T_LEN = 0;
   UEP1_T_LEN = 0;
   UEP2_T_LEN = 0;

   while(1)
   {
      if(UsbConfig)
      {
         if(TxBytes)
         {
            UART1SendByte(EP2_OutBuffer[USBBufOutPoint++]);
            TxBytes--;
            if(TxBytes==0)
               UEP2_CTRL = UEP2_CTRL & ~ MASK_UEP_R_RES | UEP_R_RES_ACK;
         }
         if(RxBytes) timeout++;
         if(!UpPoint2_Busy)
         {
            len = RxBytes;
            if(len>0)
            {
               if(len>39 || timeout>100)
               {
                  timeout = 0;
                  if(wrPtr+len > RXSIZE)
                    len = RXSIZE-wrPtr;
                  RxBytes -= len;
                  memcpy(EP2_InBuffer,&rxBuffer[wrPtr],len);
                  wrPtr+=len;
                  if(wrPtr>=RXSIZE)
                     wrPtr = 0;
                  UEP2_T_LEN = len;
                  UEP2_CTRL = UEP2_CTRL & ~ MASK_UEP_T_RES | UEP_T_RES_ACK;
                  UpPoint2_Busy = 1;
               }
            }
         }
      }
   }
}
